// Copyright 2023 Woodpecker Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package plugin

import (
	"github.com/urfave/cli/v3"
)

const (
	ForgeTypeGitea     = "gitea"
	ForgeTypeForgejo   = "forgejo"
	ForgeTypeGitHub    = "github"
	ForgeTypeGitLab    = "gitlab"
	ForgeTypeBitbucket = "bitbucket"
)

// Forge defines metadata for integration with a forge.
type Forge struct {
	Type string `json:"type,omitempty"`
	URL  string `json:"url,omitempty"`

	// Deprecated: Please use URL instead.
	Link string `json:"link,omitempty"`
}

func forgeFlags() []cli.Flag {
	return []cli.Flag{
		&cli.StringFlag{
			Name:  "forge.type",
			Usage: "forge type (gitea, forgejo, github, gitlab, bitbucket)",
			Sources: cli.EnvVars(
				"CI_FORGE_TYPE",
			),
		},
		&cli.StringFlag{
			Name:    "forge.url",
			Aliases: []string{"forge.link"},
			Usage:   "forge url",
			Sources: cli.EnvVars(
				"CI_FORGE_URL",
				"CI_FORGE_LINK",
			),
		},
	}
}

func forgeFromContext(c *cli.Command) Forge {
	return Forge{
		Type: c.String("forge.type"),
		Link: c.String("forge.url"),
		URL:  c.String("forge.url"),
	}
}
